@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/backend.c,v 1.150.2.9 2002/09/06 01:43:57 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/backend.c,v 1.150.2.12 2002/10/10 01:45:55 kurt Exp $ */
 /*
  * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -736,16 +736,9 @@ backend_check_controls(
 	Operation *op,
 	const char **text )
 {
-	LDAPControl **ctrls;
-	ctrls = op->o_ctrls;
-	if( ctrls == NULL ) {
-		return LDAP_SUCCESS;
-	}
+	LDAPControl **ctrls = op->o_ctrls;
 
-	if ( be->be_controls == NULL ) {
-		*text = "control unavailable in context";
-		return LDAP_UNAVAILABLE_CRITICAL_EXTENSION;
-	}
+	if( ctrls == NULL ) return LDAP_SUCCESS;
 
 	for( ; *ctrls != NULL ; ctrls++ ) {
 		if( (*ctrls)->ldctl_iscritical &&
@@ -873,6 +866,15 @@ backend_check_restrictions(
 			return LDAP_CONFIDENTIALITY_REQUIRED;
 		}
 
+
+		if( op->o_tag == LDAP_REQ_BIND && opdata == NULL ) {
+			/* simple bind specific check */
+			if( op->o_ssf < ssf->sss_simple_bind ) {
+				*text = "confidentiality required";
+				return LDAP_CONFIDENTIALITY_REQUIRED;
+			}
+		}
+
 		if( op->o_tag != LDAP_REQ_BIND || opdata == NULL ) {
 			/* these checks don't apply to SASL bind */
 
@@ -1111,13 +1113,12 @@ Attribute *backend_operational(
 	 * and the backend supports specific operational attributes, 
 	 * add them to the attribute list
 	 */
-#ifdef SLAPD_SCHEMA_DN
 	if ( opattrs || ( attrs &&
 		ad_inlist( slap_schema.si_ad_subschemaSubentry, attrs )) ) {
 		*ap = slap_operational_subschemaSubentry( be );
 		ap = &(*ap)->a_next;
 	}
-#endif
+
 	if ( ( opattrs || attrs ) && be && be->be_operational != NULL ) {
 		( void )be->be_operational( be, conn, op, e, attrs, opattrs, ap );
 	}
