@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/daemon.c,v 1.211.2.8 2002/09/17 02:26:40 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/daemon.c,v 1.211.2.11 2002/10/10 01:40:36 kurt Exp $ */
 /*
  * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -49,7 +49,7 @@ Listener **slap_listeners = NULL;
 
 static ber_socket_t wake_sds[2];
 
-#ifdef NO_THREADS
+#if defined(NO_THREADS) || defined(HAVE_GNU_PTH)
 static int waking;
 #define WAKE_LISTENER(w) \
 ((w && !waking) ? tcp_write( wake_sds[1], "0", 1 ), waking=1 : 0)
@@ -581,6 +581,8 @@ static int slap_open_listener(
 		return rc;
 	}
 
+	l.sl_url = NULL;
+
 #ifndef HAVE_TLS
 	if( ldap_pvt_url_scheme2tls( lud->lud_scheme ) ) {
 #ifdef NEW_LOGGING
@@ -859,10 +861,21 @@ static int slap_open_listener(
 
 	slap_free_listener_addresses(psal);
 
+	if ( l.sl_url == NULL )
+	{
+#ifdef NEW_LOGGING
+		LDAP_LOG( CONNECTION, RESULTS, 
+			"slap_open_listener: failed on %s\n", url, 0, 0 );
+#else
+		Debug( LDAP_DEBUG_TRACE,
+			"slap_open_listener: failed on %s\n", url, 0, 0 );
+#endif
+		return -1;
+	}
 
 #ifdef NEW_LOGGING
 	LDAP_LOG( CONNECTION, RESULTS, 
-			"slap_open_listener: daemon initialzed %s\n", l.sl_url, 0, 0 );
+		"slap_open_listener: daemon initialized %s\n", l.sl_url, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE, "daemon: initialized %s\n",
 		l.sl_url, 0, 0 );
@@ -1315,7 +1328,7 @@ slapd_daemon_task(
 		if( FD_ISSET( wake_sds[0], &readfds ) ) {
 			char c[BUFSIZ];
 			tcp_read( wake_sds[0], c, sizeof(c) );
-#ifdef NO_THREADS
+#if defined(NO_THREADS) || defined(HAVE_GNU_PTH)
 			waking = 0;
 #endif
 			continue;
