@@ -119,10 +119,15 @@ modify_check_duplicates(
 					mr, &nmods[ i ], &nvals[ j ] );
 				if ( rc != LDAP_SUCCESS ) {
 					nmods[ i + 1 ].bv_val = NULL;
+					*text = textbuf;
+					snprintf( textbuf, textlen,
+						"%s: matching rule failed",
+						ad->ad_cname.bv_val );
 					goto return_results;
 				}
 	
 				if ( match == 0 ) {
+					*text = textbuf;
 					snprintf( textbuf, textlen,
 						"%s: value #%d provided more than once",
 						ad->ad_cname.bv_val, i );
@@ -142,10 +147,15 @@ modify_check_duplicates(
 				mr, &nmods[ i ], &nmods[ j ] );
 			if ( rc != LDAP_SUCCESS ) {
 				nmods[ i + 1 ].bv_val = NULL;
+				*text = textbuf;
+				snprintf( textbuf, textlen,
+					"%s: matching rule failed",
+					ad->ad_cname.bv_val );
 				goto return_results;
 			}
 
 			if ( match == 0 ) {
+				*text = textbuf;
 				snprintf( textbuf, textlen,
 					"%s: value #%d provided more than once",
 					ad->ad_cname.bv_val, j );
@@ -183,10 +193,15 @@ modify_check_duplicates(
 					ad->ad_type->sat_syntax,
 					mr, &nmods[ i ], &asserted );
 				if ( rc != LDAP_SUCCESS ) {
+					*text = textbuf;
+					snprintf( textbuf, textlen,
+						"%s: matching rule failed",
+						ad->ad_cname.bv_val );
 					goto return_results;
 				}
 
 				if ( match == 0 ) {
+					*text = textbuf;
 					snprintf( textbuf, textlen,
 						"%s: value #%d provided more than once",
 						ad->ad_cname.bv_val, j );
@@ -281,7 +296,6 @@ modify_add_values(
 		}
 
 	} else {
-
 		/*
 		 * The original code performs ( n ) normalizations 
 		 * and ( n * ( n - 1 ) / 2 ) matches, which hide
@@ -310,8 +324,6 @@ modify_add_values(
 		 */
 
 		int		rc;
-		const char	*text = NULL;
-		char		textbuf[ SLAP_TEXT_BUFLEN ] = { '\0' };
 
 		if ( mod->sm_bvalues[ 1 ].bv_val == 0 ) {
 			if ( a != NULL ) {
@@ -319,7 +331,7 @@ modify_add_values(
 				int		i;
 
 				rc = value_normalize( mod->sm_desc, SLAP_MR_EQUALITY,
-					&mod->sm_bvalues[ 0 ], &asserted, &text );
+					&mod->sm_bvalues[ 0 ], &asserted, text );
 
 				if ( rc != LDAP_SUCCESS ) {
 					return rc;
@@ -330,10 +342,14 @@ modify_add_values(
 
 					rc = value_match( &match, mod->sm_desc, mr,
 						SLAP_MR_VALUE_SYNTAX_MATCH,
-						&a->a_vals[ i ], &asserted, &text );
+						&a->a_vals[ i ], &asserted, text );
 
 					if( rc == LDAP_SUCCESS && match == 0 ) {
 						free( asserted.bv_val );
+						*text = textbuf;
+						snprintf( textbuf, textlen,
+							"modify/%s: %s: value #0 already exists",
+							op, mod->sm_desc->ad_cname.bv_val, 0 );
 						return LDAP_TYPE_OR_VALUE_EXISTS;
 					}
 				}
@@ -342,7 +358,7 @@ modify_add_values(
 		} else {
 			rc = modify_check_duplicates( mod->sm_desc, mr,
 					a ? a->a_vals : NULL, mod->sm_bvalues,
-					&text, textbuf, sizeof( textbuf ) );
+					text, textbuf, textlen );
 	
 			if ( rc != LDAP_SUCCESS ) {
 				return rc;
@@ -454,6 +470,10 @@ modify_delete_values(
 
 			if ( rc != LDAP_SUCCESS ) {
 				free( asserted.bv_val );
+				*text = textbuf;
+				snprintf( textbuf, textlen,
+					"%s: matching rule failed",
+					mod->sm_desc->ad_cname.bv_val );
 				goto return_results;
 			}
 
